/**************************************************************************
*  Copyright (c) 2015 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  25.01.2015  mifi  First Version.
*  13.03.2016  mifi  Renamed to web_asp.c.
*  29.12.2018  mifi  Renamed to web_ssi.c.
**************************************************************************/
#define __WEB_SSI_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <time.h>

#include "tcts.h"
#include "ipstack.h"
#include "ipweb.h"

#include <pro/uhttp/uhttpd.h>
#include <pro/uhttp/streamio.h>

#if !defined(IP_WEB_SSI_EXT_CST) 
#define _IP_WEB_SSI_EXT_CST   0
#else
#define _IP_WEB_SSI_EXT_CST   IP_WEB_SSI_EXT_CST
#endif

#if (_IP_WEB_SSI_EXT_CST >= 1)
#include "web_ssi_ext_cst.h"
#endif

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  sys_ip_addr                                                          */
/*                                                                       */
/*  In    : hs                                                           */
/*  Out   : none                                                         */
/*  Return: 0 = OK / -1 = ERROR                                          */
/*************************************************************************/
static int sys_ip_addr (HTTPD_SESSION *hs)
{
   char String[16];
   
   htoa(IPIfaceAddrGet(), String, sizeof(String));
   s_puts(String, hs->s_stream);

   s_flush(hs->s_stream);

   return(0);
} /* sys_ip_addr */

/*************************************************************************/
/*  sys_ip_mask                                                          */
/*                                                                       */
/*  In    : hs                                                           */
/*  Out   : none                                                         */
/*  Return: 0 = OK / -1 = ERROR                                          */
/*************************************************************************/
static int sys_ip_mask (HTTPD_SESSION *hs)
{
   char String[16];
   
   htoa(IPIfaceMaskGet(), String, sizeof(String));
   s_puts(String, hs->s_stream);

   s_flush(hs->s_stream);

   return(0);
} /* sys_ip_mask */

/*************************************************************************/
/*  sys_ip_gate                                                          */
/*                                                                       */
/*  In    : hs                                                           */
/*  Out   : none                                                         */
/*  Return: 0 = OK / -1 = ERROR                                          */
/*************************************************************************/
static int sys_ip_gate (HTTPD_SESSION *hs)
{
   char String[16];
   
   htoa(IPIfaceGWGet(), String, sizeof(String));
   s_puts(String, hs->s_stream);

   s_flush(hs->s_stream);

   return(0);
} /* sys_ip_gate */

/*************************************************************************/
/*  sys_dhcp_ip                                                          */
/*                                                                       */
/*  In    : hs                                                           */
/*  Out   : none                                                         */
/*  Return: 0 = OK / -1 = ERROR                                          */
/*************************************************************************/
static int sys_dhcp_ip (HTTPD_SESSION *hs)
{
   char String[16];
   
   htoa(IPDhcpServerGet(), String, sizeof(String));
   s_puts(String, hs->s_stream);

   s_flush(hs->s_stream);

   return(0);
} /* sys_dhcp_ip */

/*
 * SSI variable list
 */
static const SSI_EXT_LIST_ENTRY SSIList[] =
{
   {"sys_ip_addr",          sys_ip_addr},
   {"sys_ip_mask",          sys_ip_mask},
   {"sys_ip_gate",          sys_ip_gate},
   {"sys_dhcp_ip",          sys_dhcp_ip},

#if (_IP_WEB_SSI_EXT_CST >= 1)
   WEB_SSI_EXT_CST
#endif   
      
   {NULL, NULL}
};

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  HttpSsiParseExt                                                      */
/*                                                                       */
/*  In    : hs, buf, len                                                 */
/*  Out   : none                                                         */
/*  Return: 0 = OK / -1 = Error                                          */
/*************************************************************************/
int HttpSsiParseExt (HTTPD_SESSION *hs, const char *buf, int len)
{
   int                  res = -1;
   SSI_EXT_LIST_ENTRY *pList = (SSI_EXT_LIST_ENTRY*)SSIList;
   
   while(pList->Var != NULL)
   {
      if (0 == strncasecmp(pList->Var, buf, (size_t)len))
      {
         res = pList->pFunc(hs);
         break;
      }
      pList++;
   }
   
   return(res);
} /* HttpSsiParseExt */

/*** EOF ***/


